%% Attractive tests -- can use solveBethe for the innner solver.
N = 5;  % number of nodes
T = 1; % trials

for t = 1:T
    % Construct unbiased parameters
    eta = 2 * rand(N, 1) - 1;
    J   = triu(rand(N), 1);
    J   = J + J';
    
    %% Discrete solver
    [theta, W] = mooijToUs(eta, J)
    
    [cyc, Nconstr] = completeChordlessCycles(N);
    
    % Local polytope via discretization
    [mu, xi, twoMargs, energy, complexity, exact] = solveBethe(theta, W, 'epsilon', 0.01);    
    
    %% Dual decomposition with cycle and marginal polytopes and discrete
    %% inner solver    
    cycleHist = cycleBethe(theta, W, cyc, Nconstr);    
    margHist  = margBethe(theta, W);

    %% Frank Wolfe
    
    % Construct the solver object.
    lfw = LocalBetheFrankWolfe(theta, W);    
    % Initialize to the uniform distribution.
    lfw.initUnif();
    [~, ~, lfwHist, ~] = lfw.run();    
    
    % CycleBetheFrankWolfe(theta, W, cyc) where cyc is
    %  - 'K' -- complete graph
    %  - 'C' -- cycle graph
    %  - or return value of chordlessCycles for graph of arbitrary topology
    cfw = CycleBetheFrankWolfe(theta, W, 'K');
    cfw.initUnif();
    [~, ~, cfwHist, ~] = cfw.run();    
    
    mfw = MargBetheFrankWolfe(theta, W);
    mfw.initUnif();
    [~, ~, mfwHist, ~] = mfw.run();
    
end
