function [gam, complexity] = bestMesh(theta, W, epsilon, verbose, impABflag)
% [gam, complexity] = bestMesh(theta, W, epsilon, verbose, impABflag)
%
%   NOTE: JUST USE ADAPTIVEMINSUM. For our regime (not "too" many nodes,
%   this is the best anyways).
%
%   Try all our mesh methods (1st deriv, second deriv, Lagrangian, simple)
%   and return the mesh grid for one with smallest total points.
%
%   theta is an N-vector, W is a weight matrix (symmetric), epsilon is the
%   accuracy threshold.
%
%   gams is an N-cell array, each element is the ordered vector of mesh
%   grids in that variable; complexity is a structue with fields sumN,
%   thisN, prodN.

    if nargin < 4
        verbose = false;
    end
    if nargin<5; impABflag=false; end %impABflag=true means try new impAB improved A,B bounds
        
    SIMPLE_THRESH = 1e-10;

    % Get the gams from somewhere...
        
    [A, B] = bpbound(theta, W);    
    
    % Added this Jan 15, 2014
    if impABflag
        [A, B] = impAB(theta, W, A, B);
    end
    
%     if any(A < SIMPLE_THRESH) || any(B < SIMPLE_THRESH)
%         [gam, complexity] = fdm(theta, W, A, B, epsilon, 'adaptivesimple');
%     else
%         [gam, complexity] = fdm(theta, W, A, B, epsilon, 'adaptiveminsum');
%     end
    
    [gam, complexity] = fdm(theta, W, A, B, epsilon, 'adaptiveminsum');

    % Prevent singletons    
    for n = 1:length(A)
        % Artificially sandwich
        if length(gam{n}) == 1
            if gam{n} ~= A(n)
                gam{n} = [A(n) gam{n}];
            elseif gam{n} ~= 1 - B(n)
                gam{n} = [gam{n} 1 - B(n)];
            else
                error('blah');
            end
        end                
    end
    
    %[A, B] = MKNew_mex(theta, W);    

    %[A, B] = BBPNew(theta, W);
    %fmMethods = { 'simple', 'minsum', 'adaptivesimple', 'adaptiveminsum' };
    
    % Stupid stuff; ignore the cruft and unnecessary brackets.
    
    %i = 4;   
        
    if complexity.thisN >= 1000
        warning('Problem may have high complexity:');
        complexity
%         for i=1:length(thisN{i})
%             fprint('Number of discrete points in dimension %3d: %6d',i,thisN{i});
%         end
    end
    
    if verbose
%        disp(['Methods: ' fmMethods]);
%        disp(['sumN: ' sumN]);
%        disp(['prodN: ' prodN]);
%        fprintf(1, 'Best method: %s, sumN = %d\n', fmMethods{i}, sumN{i});
    end        

end

