function [cyc, Nconstr] = completeChordlessCycles(N)
% cyc = completeChordlessCycles(N) lists all chordless cycles in N.
%
%   cyc is a structure array; each entry has the hierarchy
%   cyc(n).
%       - edges : edges in cycle n; Nedges x 2 matrix
%       - F(i)  : structure array
%           - idx       : index of Lagrange multiplier; guaranteed to be unique.
%           - edges     : edges in F; Nedges x 2 matrix
%           - compEdges : edges in C \ F; Nedges x 2 matrix

    triplets  = combnk(1:N, 3);
    Ntriplets = size(triplets, 1);
        
    for n = 1:size(triplets, 1)
        cyc(n).edges = combnk(triplets(n,:), 2);              
    end
    
    idx = 1;
    
    for n = 1:Ntriplets           % for all chordless cycles
        % Chordless cycle C with singleton F
        for i = 1:3            
            % Each edge of C can go into F in turn.
            edges     = cyc(n).edges(i,:);
            
            compIdxs = 1:3;
            compIdxs(i) = [];
            
            compEdges = cyc(n).edges(compIdxs,:);
            
            cyc(n).F(i) = var2struct(idx, edges, compEdges);
            
            idx = idx + 1;                        
        end
        
        % Chordless cycle C with F = C.
        cyc(n).F(4).idx       = idx;
        cyc(n).F(4).edges     = cyc(n).edges;
        cyc(n).F(4).compEdges = [];
        
        idx = idx + 1;
    end
    
    Nconstr = idx - 1;
end

