function [ bestFinal, hist ] = ddrunMinBethe(theta,W, cycleOrMarg, EPSILONS)

if nargin<4; EPSILONS=[1,0.5,0.25,0.1]; end % Default EPSILONS

% Check W is symmetric
if ~isequal(W,W'); error('W is not symmetric.\n'); beep; end

n=length(theta);
if n<3; error('n must be at least 3\n'); end
if size(W,1)~=n; error('Size mismatch between theta and W.\n'); beep; end
if ~isequal(diag(W), zeros(n,1)); error('W is not zero on main diagonal.\n'); end

% theta must be a column vec
if size(theta,1)==1; theta=theta'; end


% Simple heuristic initial step size
initStep = mean(abs(W(:)));

if strcmp('cycle', cycleOrMarg)
    [cyc, Nconstr] = chordlessCycles(W);
    cycOpts = { theta, W, cyc, Nconstr };
    fun = @cycleBethe;
elseif strcmp('marg', cycleOrMarg) || strcmp('marginal', cycleOrMarg)
    cycOpts = { theta, W };, 
    fun = @margBethe;
else
    error('Unsupported polytope');
end

cycOpts = { cycOpts{:}, ... 
        'initStep', initStep, ...
        'innerSolver', @solveMinBethe, ...
        'innerSolverOpts', { 'nTrials', 100 } };

nEPSILONS = length(EPSILONS);
hist=cell(nEPSILONS,1); best=cell(nEPSILONS,1);
nextLam=[];

for run=1:nEPSILONS
    fprintf('Starting run %d with epsilon=%5.3f...\n',run,EPSILONS(run));
    hist{run}=fun(cycOpts{:}, 'lamInit', nextLam, 'gapThresh', EPSILONS(run) / 3);

    if run==nEPSILONS && ( hist{run}(end).lagrOpt==0 ) % last run did not return integer solution, redo with best lam using integer solver
        fprintf('Final run did not return integer solution, redo with best lam using integer solver.\n');
        best=summarizeDD(hist{run}, true);
        nextLam=hist{run}(best.iBestDual).lam;
        hist{run}=fun(cycOpts{:}, 'lamInit', nextLam, 'gapThresh', EPSILONS(run) / 3);
    end
    best=summarizeDD(hist{run}, true);
    nextLam=hist{run}(best.iBestDual).lam;
end
bestFinal=best;

end

