function forEachConstraintAndEdge(cyc, Ffunc, compFunc)
% forEachConstraintAndEdge(cyc, Ffunc, compFunc)
%   Helper function for cycleBethe. Call Ffunc(F, u, v) for each (u, v) \in
%   F for each odd F \subset C for each C \in cyc and compFunc for (u, v)
%   \in C \ F similarly.

    Ncycles = length(cyc);    
    for nc = 1:Ncycles
        Nf = length(cyc(nc).F);
        NtotalEdges = size(cyc(nc).edges, 1);

        for nf = 1:Nf
            F  = cyc(nc).F(nf);                

            NFedges     = size(F.edges, 1);
            NcompEdges  = size(F.compEdges, 1);
            assert(NFedges + NcompEdges == NtotalEdges, 'Edges were not accounted for.');

            for ne = 1:NFedges
                ed = F.edges(ne,:);                    
                assert(ed(1) < ed(2), 'Violated our edge order convention!');

                % CALL THE F, ODD EDGE FUNCTION
                Ffunc(F, ed(1), ed(2));
            end

            for nce = 1:NcompEdges
                ed = F.compEdges(nce,:);
                assert(ed(1) < ed(2), 'Violated our edge order convention!');

                % CALL THE C \ F, COMPLEMENT EDGE FUNCTION
                compFunc(F, ed(1), ed(2));                   
            end % for nce = 1:NcompEdges                 
        end % for nf = 1:Nf
    end % nc = 1:Ncycles
end % forEachConstraintAndEdge(Ffunc, compFunc)
