function [A, b] = makeOvercompleteCyclePolytope(cyc, W)
% makeOvercompleteCyclePolytope  Cycle polytope in Ax \leq b form
%
%   [A, b] = makeOvercompleteCyclePolytope(nNodes, nEdges, cyc)
%   ONLY FOR BINARY PAIRWISE NOW.

    % Figure out the linearized edgeId given (u, v).
    % (Make the index matrix.)
    nNodes  = size(W, 1);
    edgeMap = triu(W, 1);
    nEdges  = nnz(edgeMap);    
    edgeMap(edgeMap ~= 0) = 1:nEdges; % in linear order    

    % Convert expression like \mu_{uv}(x, y) to a column index in the A
    % matrix, e.g. the corresponding dimension in the overcomplete vector.
    %
    % x, y \in {0, 1}.
    nOneMargs = 2 * nNodes; % assume binary pairwise
    edgeShape = [2 2 nEdges];
    function c = overcompleteCol(u, v, x, y)
        edgeId = edgeMap(u, v);
        c = nOneMargs + sub2ind(edgeShape, x+1, y+1, edgeId);
    end
    
    idx = 1;
    iVec = []; jVec = [];
    % All wVec coefficients are -1.
        
    % Both of these are called for a single 
    function markDiag(F, u, v)
        iVec(end+1:end+2) = [F.idx F.idx]; % row is the constraint index
        jVec(end+1:end+2) = [overcompleteCol(u, v, 0, 0) overcompleteCol(u, v, 1, 1)];        
    end

    function markOff(F, u, v)
        iVec(end+1:end+2) = [F.idx F.idx];
        jVec(end+1:end+2) = [overcompleteCol(u, v, 1, 0) overcompleteCol(u, v, 0, 1)];
    end
    
    forEachConstraintAndEdge(cyc, @markDiag, @markOff);

    wVec = -1 * ones(1, length(iVec));
    A = sparse(iVec, jVec, wVec);
    Nconstr = size(A, 1);   
    
    b = -1 * ones(Nconstr, 1);

end

