classdef CycleBetheFrankWolfe < LocalBetheFrankWolfe
% ONLY FOR BINARY PAIRWISE        
    
    properties
        A
        b
    end

    methods        
        function this = CycleBetheFrankWolfe(theta, W, cyc, sem)
            % theta, W as in parent class.
            % cyc can be a cycle structure (retval of
            % completeChordlessCycles) or 'K' or 'C' shorthand.
                        
            this = this@LocalBetheFrankWolfe(theta, W);
            
            if exist('sem', 'var')
                this.sem = sem;
            end                        
            
            % Special case the commonly-used cycles.
            if strcmp(cyc, 'K')
                cyc = completeChordlessCycles(this.N);
            elseif strcmp(cyc, 'C')
                cyc = cycleChordlessCycle(this.N);
            end
            
            [Aeq, beq] = makeOvercompleteLocalPolytope(W, 2);
            [this.A, this.b]     = makeOvercompleteCyclePolytope(cyc, W);
            
            Neq   = length(beq);
            Nineq = length(this.b);                      
            
            this.model.A   = [Aeq ; this.A];
            this.model.rhs = [beq ; this.b];
            this.model.sense = [repmat('=', Neq, 1) ; repmat('<', Nineq, 1)];
        end
        
        function initRand(this)
            % Rejection sample
            while true
                this.mu = subWtRandomInitOvercomplete(this.W);
                
                if this.A * this.mu <= this.b
                    % Accept
                    break;
                end
                
                if this.print
                    disp('Reject');
                end
                
            end            
        end
                
    end    
end

