classdef LocalBetheFrankWolfe < BetheFrankWolfe
    properties
        model
        sem
        params = struct('OutputFlag', 0);
    end
    
    methods
        
        function this = LocalBetheFrankWolfe(theta, W, sem)
            % theta, W as in parent class.
            % cyc can be a cycle structure (retval of
            % completeChordlessCycles) or 'K' or 'C' shorthand.
            this = this@BetheFrankWolfe(theta, W);
            
            if exist('sem', 'var')
                this.sem = sem;
            end                                                
            
            [Aeq, this.model.rhs] = makeOvercompleteLocalPolytope(W, 2);
            this.model.A = sparse(Aeq);
            Neq = length(this.model.rhs);
            this.model.sense = repmat('=', Neq, 1);            
        end
        
        function target = solveSubLP(this, g)
            this.model.obj = g;
            
            result = gurobi(this.model, this.params);
            assert(strcmp(result.status, 'OPTIMAL'), 'solveSubLP not solved to optimality!');

            target = result.x;
        end
        
        function initMAP(this, mix)
            m = this.model;
            m.obj = this.thetaOvercomplete;
            
            result = gurobi(m, this.params);
            assert(strcmp(result.status, 'OPTIMAL'), 'solveSubLP not solved to optimality!');
            
            muMAP = result.x;
            
            if ~exist('mix', 'var')
               mix = 1; 
            end
            
            this.initUnif();
            this.mu = (1 - mix) * this.mu + mix * muMAP;            
        end
        
        function initRand(this)
            while true
                this.mu = wtRandomInitOvercomplete(this.W);                                
                if ~this.onBoundary()
                    % Accept
                    break;
                end
            end
            
            % Check that we are in the polytope
%             Ax = this.model.A * this.mu;
%             assertElementsAlmostEqual(Ax, this.model.rhs);
        end
        
        function exportPolytope(this, filename)
            Nconstr = length(this.model.rhs);
            
            constr = full(this.model.A);
            rhs = this.model.rhs';
            dir = this.model.sense;
            % Change to an easier format; no cell-arrays. To translate back
            % in the R part.
%             dir(dir == '=') = 0;
%             dir(dir == '<') = 1;
                        
            save(filename, 'constr', 'rhs', 'dir', '-v4');
        end
        
    end
    
end

