classdef MargBetheFrankWolfe < BetheFrankWolfe
        
    properties
        extremes
    end
    
    methods
        
        function this = MargBetheFrankWolfe(theta, W)
            this = this@BetheFrankWolfe(theta, W);
            this.extremes = sparseMargExtreme(W);            
        end
        
        function target = solveSubLP(this, g)
            objs = sum(bsxfun(@times, g, this.extremes), 1);        
            [fMin, iMin] = min(objs);        
            target = this.extremes(:,iMin);
        end
        
        function initMAP(this, mix)
            objs = sum(bsxfun(@times, this.thetaOvercomplete, this.extremes), 1);
            [fMin, iMin] = min(objs); 
            muMAP = this.extremes(:,iMin);
            
            if ~exist('mix', 'var')
                mix = 1;
            end
            
            this.initUnif();
            this.mu = (1 - mix) * this.mu + mix * muMAP;            
        end
        
        function initKMAPs(this, K, diversity)
            objs = sum(bsxfun(@times, this.thetaOvercomplete, this.extremes), 1);            
            
            [~, sortedIMin] = sort(objs);
            % Greedily add sets that are more than 1 Hamming distance away
            k = 0;
            i = 1;
            D = size(this.extremes, 1);
            workingVerts = zeros(D, K);
            reject = false;
            while k < K    
                ii = sortedIMin(i);
                candidateVert = this.extremes(:,ii);
                for j = 1:k
                    dst = norm(workingVerts(:,j) - candidateVert, 1);
                    % If candidateVert is only Hamming distance of 1 away
                    % from any vertex in the working set, reject and move
                    % on.
                    if dst <= diversity
                        reject = true;
                        break;
                    end
                end
                
                if reject
                    continue;
                else
                    % Didn't reject, so we add it to working
                    k = k + 1;
                    workingVerts(:,k) = candidateVert;
                end           
                i = i + 1;
            end
            
            mix = randfixedsum(K, 1, 1, 0, 1);
            this.mu = workingVerts * mix;
            
%             figure;
%             plot(this.mu);
            
            % Fixup: Move away from 0 and 1.
            this.mu(this.mu == 0) = 1e-6;
            this.mu(this.mu == 1) = 1 - 1e-6;

            if this.plotInit
                figure(this.ph);
                plot(this.mu);
            end            
        end
        
        function initKRand(this, K)
            nExtreme = size(this.extremes, 2);
            selIdxs = randsample(1:nExtreme, K);
            
            workingVerts = this.extremes(:,selIdxs);            
            
            mix = randfixedsum(K, 1, 1, 0, 1);
            this.mu = workingVerts * mix;

            % Fixup: Move away from 0 and 1.
            this.mu(this.mu == 0) = 1e-6;
            this.mu(this.mu == 1) = 1 - 1e-6;
            
            if this.plotInit
                figure(this.ph);
                plot(this.mu);
            end
        end
        
        function initRandSimplex(this)
            nExtremes = size(this.extremes, 2);
            mix = randfixedsum(nExtremes, 1, 1, 0, 1);
            
            this.mu = this.extremes * mix;
            if this.plotInit
                figure(this.ph);
                plot(this.mu);
            end
        end
        
        function initRandWrong(this)
            nExtremes = size(this.extremes, 2);
            mix = rand(nExtremes, 1);
            mix = mix ./ sum(mix);
            
            this.mu = this.extremes * mix;
            plot(this.mu)
        end
        
        function initRandRound(this, mix)
            [~, oneMargs, twoMargs] = wtRandomInitOvercomplete(this.W);
            mu = oneMargs(2,:);
            [~, ~, muEdge, ~, ~, ~] = roundToMarg(mu, twoMargs, this.W);
            
            this.initUnif();
            this.mu = (1 - mix) * this.mu + mix * muEdge;
            
%             plot(this.mu)
        end
        
    end
    
end

