function F = betheOvercomplete(mu, theta, degs, N, nEdges)
% betheOvercomplete  Evaluate BFE in overcomplete form.
%
%   F = betheOvercomplete(mu, theta, degs, N, nEdges) evaluates the BFE of
%   potentials theta at the point mu. Both mu and theta are vectors in the
%   overcomplete space as described in sparseMargExtreme.
%
%   Note that degs may not actually be degrees--they are just Bethe
%   counting numbers.    
            
    [muOne, muTwo] = unpackOvercomplete(mu, N, nEdges);
    [thetaOne, thetaTwo] = unpackOvercomplete(theta, N, nEdges);
    
    eOne = -sum(vec(muOne .* thetaOne));
    eTwo = -sum(vec(muTwo .* thetaTwo));
    
    % mu log on
    mlOne = muOne .* log(muOne);
    mlOne(muOne == 0) = 0;
    
    mlTwo = muTwo .* log(muTwo);
    mlTwo(muTwo == 0) = 0;
    
    % \sum_i (\delta_i - 1) \sum_{x_i} \mu_i(x_i) \log \mu_i(x_i)
%     sOne = -sum(bsxfun(@times, degs(:)' - 1, sum(mlOne, 1)));
    
    sOne = -sum((degs(:)' - 1) .* sum(mlOne, 1));
%     assertElementsAlmostEqual(sOne, sOneNoBsx);
    
    % NOTE THE ASYMMETRY! The net contribution of sOne is NEGATIVE.
    fOne = eOne + sOne;
    
    % -\sum_{ij} \sum_{x_i x_j} b_{ij}(x_i, x_j) \log b_{ij}(x_i, x_j)
    sTwo = -sum(mlTwo(:));
    % The net contribution of sTwo is POSITIVE.
    fTwo = eTwo - sTwo;
       
    F = fOne + fTwo;
    
end

