function [cyc, Nconstr] = chordlessCycles(W)
% chordlessCycles  Find chordless cycles in arbitrary graph
%
%   [cyc, Nconstr] = chordlessCycles(W) uses Delbert Dueck's code
%   http://www.cs.columbia.edu/~delbert/ and stores the results in a format
%   for cycleBethe.
%
%   This code has exponential running time.
    N    = size(W, 1);    
    holes = findholeofsizeD(W, 3:N);
    Nholes = length(holes);
    edges  = cell(Nholes, 1);
    
    for hi = 1:Nholes
        h  = holes{hi};        
        for i = 1:(length(h)-1)
            edges{hi}(end+1,1:2) = h(i:i+1);
        end
        
        % Canonicalize: i < j
        for i = 1:size(edges{hi}, 1)
            edges{hi}(i,:) = sort(edges{hi}(i,:));
        end
        % Canonicalize: sort rows
        edges{hi} = sortrows(edges{hi});
    end
    
    edges = uniquecell(edges);
    
    fidx = 1;
    for hi = 1:length(edges)
        cyc(hi).edges = edges{hi};
        Nedges = length(edges{hi});
        
        ci = 1;
        
        % For each odd k
        for k = 1:2:Nedges
            inFidxs = combnk(1:Nedges, k);
            nF = size(inFidxs, 1);
            for fi = 1:nF
                edgeMask = false(1, Nedges);
                edgeMask(inFidxs(fi,:)) = true;
                
                st.idx       = fidx;
                st.edges     = edges{hi}(edgeMask,:);
                st.compEdges = edges{hi}(~edgeMask,:);
                
                cyc(hi).F(ci) = st;
                ci   = ci + 1;
                fidx = fidx + 1;
            end
            
        end
    end
    
    Nconstr = fidx - 1;    

end

