%% matlabpool
matlabpool open 12;
nRestarts = 12; 

%% Load of one David's 10 node graphs
load('trialsComplete10.mat');


T = size(graph, 2);
N   = ref(4, t);
obj = graph(:, t);
G   = make_complete_graph(N, obj);

nEdges = sum(vec(G.Weights ~= 0)) / 2;
degs = sum(G.Weights ~= 0, 1);

%% Frank-Wolfe 
x = cell(nRestarts, 1);
f = zeros(1, nRestarts);
gaps = zeros(1, nRestarts);

hist = cell(nRestarts, 1);

parfor n = 1:nRestarts
    %% One run
    [x{n}, f(n), hist{n}] = margBetheFrankWolfe(G.Fields, G.Weights, 'print', false, 'linesearch', 'fminbnd', 'linesearchUb', 1.0, 'plotLine', false);
    gaps(n) = hist{n}(end).gap;
end

fStd = std(f)
[fMin, iMin] = min(f)
gaps(iMin);

bestHist = hist{iMin};

frankWolfePrimalHist = [hist{iMin}.f];

matlabpool close;

save(sprintf('cp/fw_%d.mat', t));

%figure;
%plot(frankWolfePrimalHist);
%title(sprintf('Frank-Wolfe Primal vs. Iter, final = %g', fMin));
%print('-depsc', sprintf('figures/frankWolfe_primal_iter_%d.eps', t));
%% close;

% Sanity check
% [muBiased, twoMargsBiased] = overcompleteToBiased(x, N, nEdges);

%% minBethe 
%hist = margBethe(G.Fields, G.Weights, 'innerSolver', @solveMinBethe, 'innerSolverOpts', { 'nTrials', nRestarts, 'parallelRestarts', true });
%best = summarizeDD(hist, true) % second argument: ignoreValidity.
%
%figure;
%plot([hist.primal]);
%title(sprintf('margBethe minBethe Primal vs. DD Iter, final = %g', best.primal));;
%print('-depsc', sprintf('figures/margBethe_minBethe_primal_iter_%d.eps', t));
%

%% hak 
%opts = {'method', 'HAK', 'restarts', nRestarts, 'parallelRestarts', true};
%hist = margBethe(G.Fields, G.Weights, 'innerSolver', @solveDAI, 'innerSolverOpts', opts); 
%best = summarizeDD(hist, true)
%
%save(sprintf('cp/hak_%d.mat', t));
%
%figure;
%plotDDMargHist(hist);
%title(sprintf('margBethe HAK Primal vs. DD Iter, final = %g', best.primal));;
%print('-depsc', sprintf('figures/margBethe_HAK_primal_iter_%d.eps', t));
%
%% Test the derivatives.
% 
% theta = biasedToOvercomplete(G.Fields, G.Weights);
% 
% opts = optimset('GradObj', 'on', 'Algorithm', 'interior-point', ...        
%     'Display', 'off', ...
%     'FinDiffType', 'central', ...
%     'DerivativeCheck', 'on');
% 
% [~, ~, wVec] = findUT(G.Weights);
% nEdges = length(wVec);
% 
% pr.x0 = packOvercomplete(0.5 * ones(2, N), 0.25 * ones(2, 2, nEdges));
% pr.objective = makeBetheFmincon(theta, degs, N, nEdges);
% pr.solver    = 'fmincon';    
% pr.options   = opts;
% [pr.Aeq, pr.beq] = makeOvercompleteLocalPolytope(G.Weights, 2);
% 
% [x,fval,exitflag] = fmincon(pr);
