function [muMin, fMin, hist] = margBetheFrankWolfe(theta, W, varargin)
% margBetheFrankWolfe

% TODO: Work on linesearch.
% TODO: Refactor into helper functions (it's gotten complicated enough.)
% TODO: Add cycle inequalities (after helper functions; see Sontag)

    p = inputParser;
    p.KeepUnmatched = true; % Enable extraneous options.
    p.addRequired('theta', @isnumeric);
    p.addRequired('W');
        
    % If true, must change of the given value must be bounded.
    p.addParamValue('convFun', true);
    p.addParamValue('convX',   true);
    
    p.addParamValue('TolFun',  1e-6);    
    p.addParamValue('TolX',    1e-6);        
    
    % Options: fminbnd, armijo, Direct, others.
    % More sophisticated armijo linesearchers?
    p.addParamValue('linesearch', 'fminbnd')    
    % Avoid moving all the way to the extreme point.
    p.addParamValue('linesearchUb', 1.0);
    % Scaled to L_2 distance between current iterate and minimizer vertex.
    p.addParamValue('exhaustiveStepSz', 1e-4);     % FAIL.
    p.addParamValue('exhaustiveSteps', 1e4);
    
    % uniform or randomInterior or randomExtreme (random point in the marginal polytope)
    p.addParamValue('init', 'randomInterior');
    % TODO: Forward randomInterior
    p.addParamValue('randomExtremeStep', 0.9999);
    
    p.addParamValue('warmStartStruct', []);
    p.addParamValue('saveWarmStart', false);
    
    % Debugging options
    p.addParamValue('storeHist', true);
    p.addParamValue('print', true);
    p.addParamValue('plotLine', false);    

    p.parse(theta, W, varargin{:});            
    o = p.Results;
    
    if o.plotLine
        ph = figure;
    end

    N = length(theta);
    [extremes, D, nEdges] = sparseMargExtreme(W);        
    nExtremes = size(extremes, 2);
    degs = sum(W ~= 0, 1);        
    
    unifOne = 0.5 * ones(2, N);
    unifTwo = 0.25 * ones(2, 2, nEdges);
    unifMu  = packOvercomplete(unifOne, unifTwo);        
    
    if strcmp(o.init, 'uniform')
        % Begin from the uniform distribution. This may not be sparse, but
        % doesn't really matter so long as we keep track of the additions.
        newmu = unifMu;
    elseif strcmp(o.init, 'randomInterior')
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % TODO: Don't have polytope in A\mu \leq b form!
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%

        mix = rand(1, nExtremes);
        mix = mix ./ sum(mix);
        
        newmu = sum(bsxfun(@times, mix, extremes), 2);
        figure;
        plot(newmu);
        
        % Sample uniformly from the marginal polytope using accelerated hit
        % and run.
        %
        % See util/cprnd.m and references therein.        
%         [A, b] = makeMargPolytope(N, D, extremes);
%         newmu = cprnd(1, A, b, 'method', 'achr');
%         
%         % Double check (possibly need some fudge).
%         % TODO: May need to check boundedness away from boundary. For now,
%         % we compromise no fudge + strict inequality => far enough away?
%         assert(all(A*newmu < b), 'sampled newmu not in marginal polytope.');        
        
    elseif strcmp(o.init, 'randomExtreme')
        % NOTE: The iterates must be bounded away from an extreme point.
        % Otherwise, gradients go to infinity.
        %
        % Thus, we just move a little fudge away.
        i = randi(nExtremes, 1);
        newmu = (1 - o.randomExtremeStep)*unifMu + o.randomExtremeStep*extremes(:,i);        
    else
        error('margBetheFrankWolfe:init', 'init must be uniform, randomInterior, or randomExtreme');
    end
        
    % TODO: Track the extremes and convex combinations added.
    % (Sparse representation; see David Belanger's appendices.)
    
    thetaOvercomplete = biasedToOvercomplete(theta, W);
    
    obj  = @(x) betheOvercomplete(x, thetaOvercomplete, degs, N, nEdges);
    grad = @(x) gradBetheOvercomplete(x, thetaOvercomplete, degs, N, nEdges);
    
    % Sentinels
    % NOTE: Actual initial mu is newmu. See first line of the loop body.
    mu = 1e100 * newmu;
    newF = 1e100;
    f = 1e101;
    iter = 1;
    
%     while abs(newF - f) > o.TolFun
    while (~o.convX    || norm(newF - f, 1) > o.TolX) && ...
          (~o.convFun  || norm(newmu - mu, 1) > o.TolFun)
        mu = newmu;
        f = obj(mu);   
        
        % DOUBLE CHECK THE OVERCOMPLETE REPRESENTATION. (Correct.)
        [muBiased, twoMargsBiased] = overcompleteToBiased(mu, N, nEdges);
%         fBiased = bethe(theta, W, muBiased, twoMargsBiased);
%         assertElementsAlmostEqual(f, fBiased);
        
        g = grad(mu);       
        % HACK
%         g(g == Inf) = 1e10;
        
        % Evaluate the gradient dot each extreme point.
        objs = sum(bsxfun(@times, g, extremes), 1);        
        [fMin, iMin] = min(objs);        
        xMin = extremes(:,iMin);
        
        % Linesearch: 0 is current x, 1 is xMin
        
        arg  = @(t) (1 - t)*mu + t*xMin;
        line = @(t) obj(arg(t));         
        
        % fminbnd is not aggressive enough.

        if strcmp(o.linesearch, 'fminbnd')
            [t, tFMin, exitflag] = fminbnd(line, 0, o.linesearchUb);
            assert(exitflag == 1, 'fminbnd did not converge.');
        elseif strcmp(o.linesearch, 'exhaustive')
            lineStep = o.exhaustiveStepSz / norm(mu - xMin, 2);
%             mesh     = [0:lineStep:1 1];
            % TODO: Vectorize! For loop is JITed.
%           
            mesh = linspace(0, o.linesearchUb, o.exhaustiveSteps);
            nSteps   = length(mesh);
            fvals = zeros(nSteps, 1);
            for i = 1:nSteps
                fvals(i) = line(mesh(i));
            end
            
            [tFMin, iMin] = min(fvals);
            t = mesh(iMin);
            
            if t == 1
                warning('margBetheFrankWolfe:extreme', 'Hitting an extreme point. Gradient may diverge to infinity.');
            end
            
        elseif strcmp(o.linesearch, 'direct')
            Problem.f = line;
            bounds    = [0 o.linesearchUb];
            opts.tol  = o.TolX * 1e-2; % tighter tolerance here
            opts.maxevals = 2000;
            opts.maxits   = 100;
            [tFMin, t, directHistory] = Direct(Problem,bounds,opts);
        else
            error('margBetheFrankWolfe:linesearch', 'Unsupported linesearch method.');
        end
        
        if o.plotLine
%             clf(ph);
            figure(ph);            
            ax = gca;
            hold on;
            ezplot(ax, line, [0, 1]);
            plot(ax, t, tFMin, '-.or');
            title(sprintf('Iter = %d, min = (%g, %g)', iter, t, tFMin));
            hold off;
        end        
        
        newmu = arg(t);        
        newF  = obj(newmu);
                
        % g(x) = \left< x - s, \nabla f(x) \right>; Jaggi13 Eq. (2)
        % Note that duality gap may be meaningless for our nonconvex
        % objective.
        %
        % Substitute x -> mu, s -> xMin.
        gap = sum((mu - xMin) .* g);
        
        if o.storeHist
            hist(iter) = var2struct(f, g, mu, t, xMin, tFMin, gap);
        end
        
        if o.print
            fprintf(1, 'Iter=%d, f=%g, |g|_1=%g, gap=%g, t=%g\n', iter, f, norm(g, 1), gap, t);
        end
        
        newF = obj(newmu);        
        iter = iter + 1;
    end
    
    if ~o.storeHist
        hist = [];
    end
    
    [fMin, iMin] = min([hist.f]);    
    
    if hist(iMin).f ~= hist(end).f
        % 30 -> 20140213T022040 format
        saveFile = sprintf('cp/margBetheFrankWolfe_nonmonotonic_%s.mat', datestr(now, 30));
        warning(['margBetheFrankWolfe was not monotonic. Saved to ' saveFile]);
    end
    
    muMin = hist(iMin).mu;

end
