%% matlabpool
matlabpool open 12;
nRestarts = 12; 

%% Load of one David's 10 node graphs
load('trialsComplete10.mat');

T = size(graph, 2);
N   = ref(4, t);
obj = graph(:, t);
G   = make_complete_graph(N, obj);

nEdges = sum(vec(G.Weights ~= 0)) / 2;
degs = sum(G.Weights ~= 0, 1);

%% Export a CDL file
cdlFile = 'cdl_test.f90';
dtFile  = 'cdl_test.DT1';

thetaOvercomplete = biasedToOvercomplete(G.Fields, G.Weights);
fortBetheOvercompleteCDL(cdlFile, thetaOvercomplete, degs, N, nEdges)
fortBetheOvercompleteDT(dtFile, 1e-4, thetaOvercomplete);

%% Local polytope, R random starts
lfw = LocalBetheFrankWolfe(G.Fields, G.Weights);
lfw.exportPolytope('cp/POLYTOPE');

%% Cycle polytope, R random starts
cfw = CycleBetheFrankWolfe(G.Fields, G.Weights, 'K');
cfw.exportPolytope('cp/POLYTOPE');

%% Marginal polytope, R random starts

clear mfw;

mfw = MargBetheFrankWolfe(G.Fields, G.Weights);
mfw.initUnif();
% figure;
% mfw.initRandWrong();
[muMin, fMinUnif, myHist] = mfw.run();
fMinUnif;

% Marginal polytope, k-MAP random start
nTrials = 500;
fMins = zeros(nTrials, 1);

diversity = 3;

h = figure;

for r = 1:nTrials
    try
        mfw = MargBetheFrankWolfe(G.Fields, G.Weights);
        mfw.initKRand(20);
        mfw.ph = h;
        mfw.plotInit = true;        
        [muMin, fMins(r), myHist] = mfw.run();
    catch e
        fMins(r) = NaN;
    end
end

figure; hist(fMins);

minRandRestarts = min(fMins);

fprintf('UNIF - RANDOM STARTS: %g - %g = %g', fMinUnif, minRandRestarts, fMinUnif - minRandRestarts);


%% Marginal polytope, random start

fMins = zeros(100, 1);

for t = 1:100
    try
        mfw = MargBetheFrankWolfe(G.Fields, G.Weights);
        mfw.initRandSimplex();
%         mfw.initRandRound(0.99);
    %     lfw.ph = figure;
        mfw.plotLine = false;
        [muMin, fMins(t), myHist] = mfw.run();
    catch e
        fMins(t) = NaN;
    end
end

figure; hist(fMins);

minRandRestarts = min(fMins);

fprintf('UNIF - RANDOM STARTS: %g - %g = %g', fMinUnif, minRandRestarts, fMinUnif - minRandRestarts);

%% Local polytope, random start

fMins = zeros(100, 1);

for t = 1:100
    lfw = LocalBetheFrankWolfe(G.Fields, G.Weights);
    lfw.initRand();
%     lfw.ph = figure;
    lfw.plotLine = false;
    [muMin, fMins(t), myHist] = lfw.run();
end

figure; hist(fMins);

%% Cycle polytope, random start

fMins = zeros(100, 1);

for t = 1:100
    cfw = CycleBetheFrankWolfe(G.Fields, G.Weights, 'K');
    cfw.initRand();
%     lfw.ph = figure;
    cfw.plotLine = false;
    [muMin, fMins(t), myHist] = cfw.run();
end

figure; hist(fMins);

%% Object Oriented Frank-Wolfe
mfw = MargBetheFrankWolfe(G.Fields, G.Weights);
mfw.initMAP(0.9);
% mfw.initUnif();
mfw.ph = figure;
mfw.plotLine = true;
mfw.run();

%% Object oriented Frank-Wolfe, cycle polytope
cfw = CycleBetheFrankWolfe(G.Fields, G.Weights, 'K');
cfw.initUnif();
cfw.ph = figure;
cfw.plotLine = true;
cfw.run();

%% Local polytope
lfw = LocalBetheFrankWolfe(G.Fields, G.Weights);
lfw.initUnif();
lfw.ph = figure;
lfw.plotLine = true;
[muMin, fMin, myHist] = lfw.run();

%% Extended Gradient
meg = MargBetheExtGrad(G.Fields, G.Weights);
meg.initUnif();
meg.ph = figure;
meg.plotLine = true;
meg.run();

%% Extended Gradient, Local
leg = LocalBetheExtGrad(G.Fields, G.Weights);
leg.initUnif();
leg.ph = figure;
leg.plotLine = true;
leg.run();

%% Frank-Wolfe 
%x = cell(nRestarts, 1);
%f = zeros(1, nRestarts);
%gaps = zeros(1, nRestarts);
%
%myHist = cell(nRestarts, 1);
%
%parfor n = 1:nRestarts
%    %% One run
%    [x{n}, f(n), myHist{n}] = margBetheFrankWolfe(G.Fields, G.Weights, 'print', false, 'linesearch', 'fminbnd', 'linesearchUb', 1.0, 'plotLine', false);
%    gaps(n) = myHist{n}(end).gap;
%end
%
%fStd = std(f)
%[fMin, iMin] = min(f)
%gaps(iMin);
%
%bestHist = myHist{iMin};
%
%frankWolfePrimalHist = [myHist{iMin}.f];
%
%figure;
%plot(frankWolfePrimalHist);
%title(sprintf('Frank-Wolfe Primal vs. Iter, final = %g', fMin));
%print('-depsc', sprintf('figures/frankWolfe_primal_iter_%d.eps', t));
%% close;

% Sanity check
% [muBiased, twoMargsBiased] = overcompleteToBiased(x, N, nEdges);

%% minBethe 
%myHist = margBethe(G.Fields, G.Weights, 'innerSolver', @solveMinBethe, 'innerSolverOpts', { 'nTrials', nRestarts, 'parallelRestarts', true });
%best = summarizeDD(myHist, true) % second argument: ignoreValidity.
%
%figure;
%plot([myHist.primal]);
%title(sprintf('margBethe minBethe Primal vs. DD Iter, final = %g', best.primal));;
%print('-depsc', sprintf('figures/margBethe_minBethe_primal_iter_%d.eps', t));
%

%% hak 
opts = {'method', 'HAK', 'restarts', nRestarts, 'parallelRestarts', true};
myHist = margBethe(G.Fields, G.Weights, 'innerSolver', @solveDAI, 'innerSolverOpts', opts); 
best = summarizeDD(myHist, true)

save(sprintf('cp/hak_%d.mat', t));

figure;
plotDDMargHist(myHist);
title(sprintf('margBethe HAK Primal vs. DD Iter, final = %g', best.primal));;
print('-depsc', sprintf('figures/margBethe_HAK_primal_iter_%d.eps', t));


%% Test the Fortran
theta = biasedToOvercomplete(G.Fields, G.Weights);
% hardcoded; bad
N = 10;
nEdges = 45;

mu = makeUnifOvercomplete(N, nEdges);

F = fortBetheOvercomplete(mu, theta, degs, N, nEdges);
Ftrue = betheOvercomplete(mu, theta, degs, N, nEdges);

assertElementsAlmostEqual(F, Ftrue);

%% Test the derivatives.
% 
% theta = biasedToOvercomplete(G.Fields, G.Weights);
% 
% opts = optimset('GradObj', 'on', 'Algorithm', 'interior-point', ...        
%     'Display', 'off', ...
%     'FinDiffType', 'central', ...
%     'DerivativeCheck', 'on');
% 
% [~, ~, wVec] = findUT(G.Weights);
% nEdges = length(wVec);
% 
% pr.x0 = packOvercomplete(0.5 * ones(2, N), 0.25 * ones(2, 2, nEdges));
% pr.objective = makeBetheFmincon(theta, degs, N, nEdges);
% pr.solver    = 'fmincon';    
% pr.options   = opts;
% [pr.Aeq, pr.beq] = makeOvercompleteLocalPolytope(G.Weights, 2);
% 
% [x,fval,exitflag] = fmincon(pr);
