function [val, muK] = minMuK(W, del)
% minMuK  Find minimal edge configuration given multipliers.
%
%   [val, muK] = minMuK(W, del) returns the minimum and minimizer of the
%   second term of marginal polytope dual decomposition; see pp. 6 of bc.
    
    N = size(W, 1);
    [iVec, jVec, wVec] = findUT(W);
    nEdges = length(iVec);
    
    cVals = zeros(2^N, 1);
    for n = 1:2^N
        mu = dec2bin(n - 1, N) - '0' + 1; % To get 1 or 2.        

        % Add the components of \delta_ij(x_i, x_j) that muC says are 1.
        for ne = 1:nEdges
            i = iVec(ne);
            j = jVec(ne);
            
            % For every \mu_ij^K in the marginal polytope, exactly one
            % entry is 1. This line selects it, and adds it to cVals(n).            
            cVals(n) = cVals(n) + del(mu(i),mu(j),ne);
        end                
    end
    
    % Find the configuration with minimum cVals, and reconstruct the 2x2
    % table representation. This is setting the appropriate entry to 1 (and
    % the rest to zero) for each edge ij.
    [val, nMin] = min(cVals);
    mu = dec2bin(nMin - 1, N) - '0' + 1; % To get 1 or 2.
    muK = zeros(2,2,nEdges);
    for ne = 1:nEdges
        i = iVec(ne);
        j = jVec(ne);
        
        muK(mu(i),mu(j),ne) = 1;
    end
        
end

